# transition.coffee - transmogrification

# easier to store state than to parse rgb() syntax
currentColor = 'fff'

# quick but ugly
clamp = (num, min, max) ->
  if num < min then min
  else if num > max then max
  else num

# for quick reference
$body = $('body')
$background = $('#background')

$top = $('#top')
$bottom = $('#bottom')

$chrome = $('#top, #bottom')
$chromeText = $('#mainTitle a, #mainTitle span, #mainMenu li a')

# here is our gallery of transitions
transitions = [
  # circles
  (from, to, callback) ->
    width =  $body.width()
    height = $body.height()

    left = clamp(Math.random(), 0.3, 0.7) * width
    top =  clamp(Math.random(), 0.3, 0.7) * height
    xDist = Math.max(left, width - left)
    yDist = Math.max(top, height - top)
    minRadius = Math.sqrt(xDist * xDist + yDist * yDist)

    splitComplement = de.color.splitComplement(from, to)

    firstColors = de.color.lerp(from, splitComplement, 10)
    lastColors = de.color.lerp(splitComplement, to, 9)
    lastColors.shift() # first color is a dupe
    lastColors.pop() # last color isn't exact due to color conversion
    lastColors.push(to)
    colors = firstColors.concat(lastColors)

    createCircle = (color, isFinal) ->
      $circle = $.tag(
        _: 'div',
        class: 'circle'
        style:
          backgroundColor: '#' + de.color.rgbToHex(color)
          left: left + 'px'
          top: top + 'px')

      $background.append($circle)
      $circle.animate(
          marginLeft: -minRadius
          marginTop: -minRadius
          width: minRadius * 2
          height: minRadius * 2
        1500
        'smoothStepx3'
        ->
          if isFinal
            $background.css('background-color', '#' + de.color.rgbToHex(to))
            $background.children('.circle').remove()
            callback())

    tick = ->
      createCircle(colors.shift(), colors.length == 0)
      setTimeout(tick, 80) if colors.length

    tick()
]

# this will fade out content, adjust the stage height, and trigger
# a random color transition
de.pageTransition = (options, callback) ->
  if options.background
    # do conversion work up front since we'll have to anyway
    rgbFrom = de.color.hexToRgb(currentColor)
    rgbTo = de.color.hexToRgb(options.background)

    # call a random transition
    transitions[0](
      rgbFrom
      rgbTo
      ->
        currentColor = options.background
        callback?() )

  # maybe animate text and textshadow colors
  chromeTextAnim = {}
  chromeTextAnim.color = options.chromeText if options.chromeText
  chromeTextAnim.textShadowAlpha = options.chromeTextShadowAlpha if options.chromeTextShadowAlpha
  $chromeText.animate(
    chromeTextAnim,
      duration: 2000
      easing: 'smoothStepx2')

  # maybe animate top and bottom bg and positions
  topAnim = {}
  bottomAnim = {}
  if options.chromeBackground
    topAnim.backgroundColor = bottomAnim.backgroundColor = options.chromeBackground
  if options.stageHeight
    topAnim.marginTop = bottomAnim.marginBottom = -(options.stageHeight / 2)
  chromeAnimOptions =
    duration: 2000
    easing: 'smoothStepxtreeeme'
  $top.animate(topAnim, chromeAnimOptions)
  $bottom.animate(bottomAnim, chromeAnimOptions)

